@@ -1,67 +1,24 @@
-contr.treatment <- function (n, base = 1, contrasts = TRUE)
-{
-    if (is.numeric(n) && length(n) == 1) {
-        if (n > 1)
-            levs <- 1:n
-        else stop("not enough degrees of freedom to define contrasts")
-    }
-    else {
-        levs <- n
-        n <- length(n)
-    }
-    contr <- array(0, c(n, n), list(levs, levs))
-    diag(contr) <- 1
-    if (contrasts) {
-        if (n < 2)
-            stop(gettextf("contrasts not defined for %d degrees of freedom",
-                n - 1), domain = NA)
-        if (is.character(base)){
-            base <- match(base,rownames(contr))
-            if(is.na(base)) stop("Undefined baseline category")
-        }
-        if (base < 1 | base > n)
-            stop("baseline group number out of range")
-        contr <- contr[, -base, drop = FALSE]
-    }
-    contr
-}
-
-
-contr.sum <- function (n, base = NULL, contrasts = TRUE)
-{
-    if (is.numeric(n) && length(n) == 1)
-        levs <- 1:n
-    else {
-        levs <- n
-        n <- length(n)
-    }
-    contr <- array(0, c(n, n), list(levs, levs))
-    diag(contr) <- 1
-    if (contrasts) {
-        if (n < 2)
-            stop(paste("Contrasts not defined for", n - 1, "degrees of freedom"))
-        if (is.null(base)) base <- n
-        if (is.character(base)){
-            base <- match(base,rownames(contr))
-            if(is.na(base)) stop("Undefined baseline category")
-        }
-        if (base < 1 | base > n)
-            stop("Baseline group number out of range")
-        contr <- contr[, -base, drop = FALSE]
-        contr[base,] <- -1
-    }
-    contr
-}
-
-
 dummy_labels <- function(x){
   ux <- sort(unique(as.character(x)))
   new("value.labels",ux,values=ux)
 }
 
 
-# copied from stats:contrasts and modified
-setMethod("contrasts","item",function(x,contrasts=TRUE)
+contr <- function(type,...){
+  call <- match.call()
+  contr.fun <- as.name(paste("contr",type,sep="."))
+  args <- list(n=quote(n),...,contrasts=quote(contrasts))
+  fun <- function(n,contrasts=TRUE) NULL
+  body(fun) <- as.call(c(contr.fun,args))
+  fun
+}
+
+setMethod("contrasts","ANY",function(x,contrasts=TRUE,...)stats::contrasts(x,contrasts=contrasts,...))
+
+## Copied from stats:contrasts and modified
+## Original copyright (C) 1995-2013 The R Core Team
+
+setMethod("contrasts","item",function(x,contrasts=TRUE,...)
 {
     if(measurement(x) %nin% c("nominal","ordinal"))
       warning("contrasts(x,...) called with non-categorical x")
@@ -92,7 +49,9 @@ setMethod("contrasts","item",function(x,contrasts=TRUE)
     ctr
 })
 
-# copied from stats:contrasts<- and modified
+## copied from stats:contrasts<- and modified 
+## Original copyright (C) 1995-2013 The R Core Team
+
 setMethod("contrasts<-","item",function(x,how.many,value){
     if(measurement(x) %nin% c("nominal","ordinal"))
       warning("contrasts(x,...) called with non-categorical x")
@@ -135,6 +94,69 @@ setMethod("contrasts<-","item",function(x,how.many,value){
     x
 })
 
+## copied from stats:contr.treatment and modified
+## Original copyright (C) 1995-2013 The R Core Team
+
+contr.treatment <- function (n, base = 1, contrasts = TRUE)
+{
+    if (is.numeric(n) && length(n) == 1) {
+        if (n > 1)
+            levs <- 1:n
+        else stop("not enough degrees of freedom to define contrasts")
+    }
+    else {
+        levs <- n
+        n <- length(n)
+    }
+    contr <- array(0, c(n, n), list(levs, levs))
+    diag(contr) <- 1
+    if (contrasts) {
+        if (n < 2)
+            stop(gettextf("contrasts not defined for %d degrees of freedom",
+                n - 1), domain = NA)
+        if (is.character(base)){
+            base <- match(base,rownames(contr))
+            if(is.na(base)) stop("Undefined baseline category")
+        }
+        if (base < 1 | base > n)
+            stop("baseline group number out of range")
+        contr <- contr[, -base, drop = FALSE]
+    }
+    contr
+}
+
+## copied from stats:contr.sum and modified
+## Original copyright (C) 1995-2013 The R Core Team
+
+contr.sum <- function (n, base = NULL, contrasts = TRUE)
+{
+    if (is.numeric(n) && length(n) == 1)
+        levs <- 1:n
+    else {
+        levs <- n
+        n <- length(n)
+    }
+    contr <- array(0, c(n, n), list(levs, levs))
+    diag(contr) <- 1
+    if (contrasts) {
+        if (n < 2)
+            stop(paste("Contrasts not defined for", n - 1, "degrees of freedom"))
+        if (is.null(base)) base <- n
+        if (is.character(base)){
+            base <- match(base,rownames(contr))
+            if(is.na(base)) stop("Undefined baseline category")
+        }
+        if (base < 1 | base > n)
+            stop("Baseline group number out of range")
+        contr <- contr[, -base, drop = FALSE]
+        contr[base,] <- -1
+    }
+    contr
+}
+
+## copied from MASS:contr.sdif and modified
+## Original copyright (C) Brian Ripley
+
 contr.sdif <- function (n, contrasts = TRUE)
 {
     if (is.numeric(n) && length(n) == 1) {
@@ -157,12 +179,3 @@ contr.sdif <- function (n, contrasts = TRUE)
     }
     else structure(diag(n), dimnames = list(lab, lab))
 }
-
-contr <- function(type,...){
-  call <- match.call()
-  contr.fun <- as.name(paste("contr",type,sep="."))
-  args <- list(n=quote(n),...,contrasts=quote(contrasts))
-  fun <- function(n,contrasts=TRUE) NULL
-  body(fun) <- as.call(c(contr.fun,args))
-  fun
-}
